#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
from threading import Thread

import cherrypy
from cherrypy.process import plugins
from ion.db import Db

class CherryPyPlugin(plugins.SimplePlugin):
    #it's called do_log due to cherrypy having a log method already
    def do_log(self, message):
        if self.server.context.settings.Ion.as_bool("verbose"):
            cherrypy.log(message, "[%s]" % self.key)

    def __init__(self, bus, server):
        if not hasattr(self, 'key'):
            raise ValueError('The plugin does not have a key. Please make sure that you have a class variable named "key" that uniquely identifies this plugin')
        super(CherryPyPlugin, self).__init__(bus)
        self.server = server

class CherryPyDaemonPlugin(CherryPyPlugin):
    def __init__(self, bus, server):
        super(CherryPyDaemonPlugin, self).__init__(bus, server)
        self.should_die = False
        self.thread = None
        self.store = None

    def start(self):
        self.thread = Thread(target=self.loop_execute)
        self.thread.setDaemon(True)
        self.thread.start()

    def stop(self):
        self.do_log("Killing monitor...")
        self.should_die = True
        self.do_log("Monitor dead.")

    def loop_execute(self):
        ctx = self.server.context
        while(not self.should_die):
            db = Db(self.server)
            try:
                db.connect()
                self.store = db.store

                self.execute()
            except Exception:
                cherrypy.engine.exit()
                raise
            finally:
                db.disconnect()
                self.store = None

    def execute(self):
        pass



"""Caching dictionary that uses access times to decide which objects to
flush from the cache.

When instantiating, you can set the maximum number of key/value pairs
to maintain in the cache:

    c = Cache(size=1000)

When the maximum size of the cache is reached, it will be resized
automatically to 95% of its maximum size the next time a new key is added to
the dictionary.  The least recently accessed items will be the ones that are
flushed.  A cache size of 0 implies an unlimited size.  (Cache size should
be controlled by an age parameter.)

You can also specify a maximum entry age when creating the cache:

    c = Cache(size=100, age=24*60)    # one hour

By default, entries will remain in the cache until they are bumped for space
or time.  If an attempt is made to retrieve an item that was added to the
cache before the age threshold, ExpiredError is raised (which is a subclass
of KeyError), just as if they weren't present in the cache.

Ages can be specified as numbers of seconds or as strings consisting of a
number followed by one of the following units: d,h,m,s
"""

import time, string, re

class CacheEntry(object):
    __slots__ = ("value", "ftime", "mtime")
    def __init__(self, value):
        self.set(value)
        self.ftime = 0.0

    def get(self):
        self.ftime = time.time()
        return self.value

    def set(self, value):
        self.value = value
        self.mtime = time.time()

class ExpiredError(KeyError):
    pass

class Cache(dict):
    def __init__(self,data=None,size=100,age=None, log=None):
        self.size = size
        self.requests = self.hits = 0
        self.inserts = self.unused = 0
        if isinstance(age, (str, unicode)):
            age = self._cvtage(age)
        self.age = age
        self.log = log

    def shrink(self):
        trim = max(0, int(len(self)-0.95*self.size))
        if trim:
            # sort keys by access times
            values = zip(self.ftimes(), self.keys())
            values.sort()
            for val,k in values[0:trim]:
                if val == 0.0:
                    self.unused += 1
                del self[k]

    def purge_old_entries(self):
        if self.age is None:
            return
        t = time.time()
        for k in self.keys():
            v = dict.__getitem__(self, k)
            threshold = t - self.age
            # modified or fetched in last self.age seconds?
            if threshold > v.mtime and threshold > v.ftime:
                if v.ftime == 0.0:
                    self.unused += 1
                del self[k]

    def __setitem__(self,key,val):
        self.inserts += 1
        if self.age is not None and self.requests % 1000 == 0:
            self.purge_old_entries()

        if (key not in self and self.size and len(self) >= self.size):
            self.shrink()
        dict.__setitem__(self, key, CacheEntry(val))
        if self.log:
            self.log("Setting item %s to %s" % (key, val), "Cache")

    def __getitem__(self,key):
        self.requests += 1
        item = dict.__getitem__(self, key)
        val = item.get()

        if self.age is not None:
            if self.requests % 1000 == 0:
                self.purge_old_entries()

            # check to make sure value has not expired
            if time.time()-self.age > item.mtime:
                if item.ftime == 0.0:
                    self.unused += 1
                del self[key]
                if self.log:
                    self.log("Cache miss at key %s" % key, "Cache")

                raise ExpiredError, key
        
        # if we get here there was no KeyError
        self.hits = self.hits + 1

        if self.log:
            self.log("Cache hit at key %s" % key, "Cache")

        return val

    def has_key(self,key):
        try:
            v = dict.__getitem__(self, key)
        except (KeyError,ExpiredError):
            return 0
        if self.age is not None and time.time()-self.age > v.mtime:
            return 0
        return 1

    def get(self,key,default=None):
        try:
            return self[key]
        except KeyError:
            return default

    def values(self):
        return [dict.__getitem__(self,key).get() for key in self]

    def ftimes(self):
        return [dict.__getitem__(self,key).ftime for key in self]

    def mtimes(self):
        return [dict.__getitem__(self,key).mtime for key in self]

    def items(self):
        return map(None, self.keys(), self.values())

    def copy(self):
        return self.__class__(self, self.size)

    def update(self, dict):
        for k in dict.keys():
            self[k] = dict[k]

    def stats(self):
        return {
            'hits': self.hits,
            'inserts': self.inserts,
            'requests': self.requests,
            'unused': self.unused,
            }

    def __repr__(self):
        l = []
        for k in self.keys():
            l.append("%s: %s" % (repr(k), repr(self[k])))
        return "{" + string.join(l, ", ") + "}"
    __str__=__repr__

    _apat = re.compile("([0-9]+([.][0-9]+)?)\s*([dhms])?\s*$")

    def _cvtage(self,age):
        mat = self._apat.match(age)
        if mat is None:
            raise ValueError, "invalid age spec: "+age
        n = float(mat.group(1))
        units = mat.group(3) or "s"
        if units == "s":
            pass
        elif units == "m":
            n = n * 60
        elif units == "h":
            n = n * 60*60
        elif units == "d":
            n = n * 24*60*60
        return n




#!/usr/bin/env python
# -*- coding:utf-8 -*-

u"""Ion - MVC Web Framework
authors: `Bernardo Heynemann <heynemann@gmail.com>
"""
__revision__ = "$Id$"
__docformat__ = 'restructuredtext en'
 
import sys
import os
from os.path import abspath
import optparse

from providers import *

def main():
    """ Main function - parses args and runs action """
    parser = optparse.OptionParser(usage="%prog run or type %prog -h (--help) for help", description=__doc__, version="%prog" + __revision__)
    #parser.add_option("-p", "--pattern", dest="pattern", default="*.acc", help="File pattern. Defines which files will get executed [default: %default].")

    (options, args) = parser.parse_args()

    if not args:
        print "Invalid action. for help type ion help"
        sys.exit(0)

    action = args[0]

    providers = Provider.all()

    found_provider = False

    for provider_type in providers:
        provider = provider_type()
        if provider.key != action:
            continue

        found_provider = True

        provider.execute(abspath(os.curdir), options, args[1:])

    if not found_provider:
        print "Invalid option. for help type ion help"

    sys.exit(0)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os.path import abspath, join, exists

from ConfigParser import ConfigParser, NoSectionError, NoOptionError

class Settings(object):
    def __init__(self, root_dir):
        self.root_dir = root_dir
        self.config = None

    def load(self, filename="config.ini"):
        path = abspath(join(self.root_dir, filename))
        if not exists(path):
            raise ValueError("The specified path (%s) was not found!" % filename)

        self.config = ConfigParser()
        self.config.read(path)

    def __getattr__(self, name):
        if not self.config:
            raise RuntimeError("You can't use any settings before loading a config file. Please use the load method.")

        return SettingsSection(self, name, self.config)

class SettingsSection(object):
    def __init__(self, settings, name, config):
        self.settings = settings
        self.name = name
        self.config = config

    def as_int(self, config_name):
        return int(getattr(self, config_name))

    def as_bool(self, config_name):
        value = getattr(self, config_name)
        if value is None:
            return False
        return {'true': True, 'false': False}.get(value.lower())

    def __getattr__(self, config_name):
        try:
            return self.config.get(self.name, config_name)
        except NoSectionError:
            return None
        except NoOptionError:
            return None




#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
sqlalchemy_tool.py (donated to the public domain by Nando Florestan)
================== http://code.google.com/p/webpyte

SQLAlchemy integration for CherryPy,
such that you can access multiple databases,
but only one of these databases per request or thread.

The database to be accessed is chosen by a properly configured CherryPy tool,
which takes care of the transaction (commiting it at the end, or rolling back
if an exception is raised).

Usage:
from sqlalchemy_tool import metadata, session, mapper
# Use the above metadata, session, mapper when defining your models

In your application configuration:

[/]
tools.SATransaction.on = True
tools.SATransaction.dburi = 'sqlite:///database.sqlite'
tools.SATransaction.echo = False           # this is the default
tools.SATransaction.convert_unicode = True # this is the default

If you set *echo* to True, SQLAlchemy will print the SQL statements.

*convert_unicode* should always be True and you should always
use unicode strings with SQLAlchemy!

When you are at the interpreter you can do:

    from sqlalchemy_tool import configure_session_for_app
    configure_session_for_app(your_cherrypy_app)
'''


from sqlalchemy import MetaData, create_engine, __version__ as sa_version
from sqlalchemy.orm import scoped_session, sessionmaker

if sa_version.split(".") < ["0", "5", "0"]:
    raise ImportError("Version 0.5 or later of SQLAlchemy required.")

def new_session():
    return scoped_session(sessionmaker(autoflush=True, autocommit=False))

metadata = MetaData()
session = new_session()
mapper = session.mapper

# A dict in which keys are connection strings and values are SA engine objects
_engines = {}

def configure_session(dburi='sqlite:///database.sqlite',
                      echo=False, convert_unicode=True):
    """This function is called on each request.
    Tool configuration is automatically passed to it as arguments.
    It gets a *dburi*,
    creates a corresponding SQLAlchemy engine if it doesn't exist,
    chooses the corresponding SQLALchemy engine,
    and binds the session to it.
    
    Additional arguments or configuration are used if an engine is created:
    *echo*: whether to print SQL statements, default is False.
    *convert_unicode*: default is True, so you should use unicode strings
    """
    engine = _engines.get(dburi, None) # Look up the dict.
    if engine is None:              # If missing engine, create and store it.
        _engines[dburi] = engine = \
            create_engine(dburi, echo=echo, convert_unicode=convert_unicode)
    session.bind = engine # Set engine for this thread or request

def configure_session_for_app(app, echo=False, convert_unicode=True):
    '''Useful when you are at the interpreter, or whenever you are outside
    a request and need to bind the session in the current thread.
    Assuming app is configured with a SQLAlchemy connection string,
    binds the session to the corresponding engine.
    '''
    adict = app.config.get('/', {})
    dburi = adict.get('tools.SATransaction.dburi', '')
    if not dburi:
        raise RuntimeError('This app is not configured for SATransaction.')
    configure_session(dburi, echo=echo, convert_unicode=convert_unicode)


import cherrypy
from cherrypy import request
from sys import exc_info


class SATransaction(cherrypy.Tool):
    """A tool that encloses each request handler in a SQLAlchemy transaction.
    See this module's docstring for more details.
    """
    # A list of exceptions that do not cause rollback. Extendable by apps.
    passable_exceptions=[cherrypy.HTTPRedirect] # KeyboardInterrupt, SystemExit
    
    def __init__(self):
        self._name = 'SATransaction'
        self._point = 'on_start_resource'
        self.callable = configure_session
        self._priority = 50
        # If this priority is not appropriate for both hook points, know that
        # you can set a 'priority' attribute on the actual functions.
        # Tool.priority is just a shortcut for that.
    
    def _setup(self):
        '''This method is called on each request to attach this tool's hooks,
        unless tools.SATransaction.on = False.
        
        If in a static request (using tools.staticdir or tools.staticfile),
        the transaction is certainly not needed,
        and thus is disabled to (probably) save some CPU time.
        '''
        if request.config.get('tools.staticdir.on', False) or \
            request.config.get('tools.staticfile.on', False):
                return
        cherrypy.Tool._setup(self)
        cherrypy.request.hooks.attach('on_end_resource', self.on_end_resource)
    
    def on_end_resource(self):
        """Method that is called after the CherryPy request handler.
        Tries to commit the transaction, then if an exception is raised,
        rolls back and raises the exception so the response body is changed.
        Ensures the session is reset and ready for next request.
        """
        typ, value, trace = exc_info()
        # Rollback if exception raised in request handler
        if value is not None and not typ in self.passable_exceptions:
            session.rollback()  # undoes what has been flushed
            session.expunge_all() # undoes what has not been flushed
            # deconfigure (unbind) session so it is ready for next request
            session.remove()
            return
        # Else try to commit (which can raise a new exception)
        try:
            session.flush()
            session.commit()
        except:
            session.rollback()    # undoes what has been flushed
            session.expunge_all() # undoes what has not been flushed
            raise                 # let this exception propagate
        finally:
            # deconfigure (unbind) session so it is ready for next request
            session.remove()


cherrypy.tools.SATransaction = SATransaction()




#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os
from os.path import join, abspath, dirname, splitext, split, exists
import inspect

import cherrypy
from cherrypy.lib.static import serve_file
from cherrypy.process.plugins import PIDFile

from ion.controllers import Controller
from sqlalchemy_tool import metadata, session, mapper, configure_session_for_app
from ion.context import Context
from ion.cache import Cache
from ion.fs import imp as import_module
from ion.fs import locate, is_file
from sqlalchemy.exc import DBAPIError
import logging

class ServerStatus(object):
    Unknown = 0
    Starting = 1
    Started = 2
    Stopping = 3
    Stopped = 4

class Server(object):
    import_method = __import__

    @classmethod
    def imp(cls, name):
        return import_module(name, Server.import_method)

    def __init__(self, root_dir, context=None):
        self.status = ServerStatus.Unknown
        self.root_dir = root_dir
        self.context = context or Context(root_dir=root_dir)
        self.template_filters = {}
        self.test_connection_error = None
        self.cache = None

    def start(self, config_path, non_block=False):
        self.status = ServerStatus.Starting
        self.publish('on_before_server_start', {'server':self, 'context':self.context})

        self.context.load_settings(abspath(join(self.root_dir, config_path)))
        self.cache = Cache(size=1000, age="5s", log=cherrypy.log)

        self.context.load_apps()
        
        self.apps = self.context.apps
        self.app_paths = self.context.app_paths
        self.app_modules = self.context.app_modules
        
        if self.context.settings.Ion.as_bool('debug'):
            logging.basicConfig()
            logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)
            logging.getLogger('sqlalchemy.orm.unitofwork').\
                                                    setLevel(logging.DEBUG)

        self.import_template_filters()
        self.import_controllers()

        if self.context.settings.Ion.pid_file:
            p = PIDFile(cherrypy.engine, self.context.settings.Ion.pid_file)
            p.subscribe()

        self.run_server(non_block)

        self.status = ServerStatus.Started
        self.publish('on_after_server_start', {'server':self, 'context':self.context})

    def import_template_filters(self):

        for app in self.apps:
            template_filters_module = Server.imp("%s.template_filters" %app)
            if template_filters_module:
                for name, func in inspect.getmembers(template_filters_module):
                    if inspect.isfunction(func) and not name.startswith("_"):
                        self.template_filters[name] = func

    def import_controllers(self):
        for app in self.apps:
            ctrl_module = Server.imp(app + '.controllers')
            if not ctrl_module:
                raise RuntimeError('The app %s does not have a controllers module.' % app)

    def stop(self):
        self.status = ServerStatus.Stopping
        self.publish('on_before_server_stop', {'server':self, 'context':self.context})

        cherrypy.engine.exit()
        cherrypy.server.httpserver = None

        self.apps = None
        self.app_paths = None
        self.app_modules = None

        self.status = ServerStatus.Stopped
        self.publish('on_after_server_stop', {'server':self, 'context':self.context})

    def get_server_settings(self):
        sets = self.context.settings

        return {
                   'server.socket_host': sets.Ion.host,
                   'server.socket_port': sets.Ion.as_int('port'),
                   'server.thread_pool': sets.Ion.as_int('threads'),
                   'request.base': sets.Ion.baseurl,
                   'tools.encode.on': True, 
                   'tools.encode.encoding': 'utf-8',
                   'tools.decode.on': True,
                   'tools.trailing_slash.on': True,
                   'log.screen': sets.Ion.as_bool('verbose'),
                   'tools.sessions.on': True
               }

    def get_mounts(self, dispatcher):
        sets = self.context.settings

        protocol = self.context.settings.Db.protocol
        username = self.context.settings.Db.user
        password = self.context.settings.Db.password
        host = self.context.settings.Db.host
        port = int(self.context.settings.Db.port)
        database = self.context.settings.Db.database

        conn_str = self.connstr(protocol, username, password, host, port, database)

        conf = {
            '/': {
                'request.dispatch': dispatcher,
                'tools.SATransaction.on': True,
                'tools.SATransaction.dburi':conn_str, 
                'tools.SATransaction.echo': sets.Ion.as_bool('verbose'),
                'tools.SATransaction.convert_unicode': True
            }
        }

        return conf

    def get_dispatcher(self):
        routes_dispatcher = cherrypy.dispatch.RoutesDispatcher()

        class MediaController():
            def __init__(self, apps):
                self.apps = apps

            def serve_media(self, media_url):
                for app in self.apps:
                    extension = splitext(media_url)[-1]
                    app_module = reduce(getattr, app.split('.')[1:], __import__(app))
                    path = inspect.getfile(app_module)
                    media_path = abspath(join("/".join(split(path)[:-1]), 'media', media_url))

                    if exists(media_path):
                        if extension == ".jpg":
                            content_type = "image/jpeg"
                        elif extension == ".gif":
                            content_type = "image/gif"
                        elif extension == ".png":
                            content_type = "image/png"
                        elif extension == ".js":
                            content_type = "text/javascript"
                        elif extension == ".css":
                            content_type = "text/css"
                        elif extension.startswith('.htm'):
                            content_type = "text/html"
                        else:
                            content_type = "text/plain"

                        return serve_file(media_path, content_type=content_type)

                raise cherrypy.HTTPError(404)

        for controller_type in Controller.all():
            controller = controller_type()
            controller.server = self
            controller.register_routes(routes_dispatcher)

        media_controller = MediaController(self.apps)
        routes_dispatcher.connect("media", "/media/{media_url:(.+)}", controller=media_controller, action="serve_media")

        route_name = "healthcheck"
        controller = Controller()
        controller.server = self
        routes_dispatcher.connect("healthcheck", "/healthcheck", controller=controller, action="healthcheck")

        dispatcher = routes_dispatcher
        return dispatcher

    def run_server(self, non_block=False):
        cherrypy.config.update(self.get_server_settings())
        dispatcher = self.get_dispatcher()
        mounts = self.get_mounts(dispatcher)

        self.app = cherrypy.tree.mount(None, config=mounts)

        self.context.use_db = self.test_connection()

        if not self.context.use_db:
            cherrypy.config.update({'tools.SATransaction.on': False})

        cherrypy.engine.start()
        if not non_block:
            cherrypy.engine.block()

    def test_connection(self):
        configure_session_for_app(self.app)
        try:
            session.execute("select 1 from dual")
        except DBAPIError, err:
            msg = '''\n\n============================ IMPORTANT ERROR ============================\nNo connection to the database could be made with the supplied parameters.\nPLEASE VERIFY YOUR CONFIG.INI FILE AND CHANGE IT ACCORDINGLY.\n=========================================================================\n\n'''
            cherrypy.log.error(msg, 'DB')
            self.test_connection_error = err
            return False
        return True

    def subscribe(self, subject, handler):
        self.context.bus.subscribe(subject, handler)

    def publish(self, subject, data):
        self.context.bus.publish(subject, data)

    def connstr(self, protocol, username, password, host, port, database):
        return "%s://%s:%s@%s:%d/%s" % (
            protocol,
            username,
            password,
            host,
            port,
            database
        )



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from os.path import split, abspath, join, dirname

from jinja2 import Environment, FileSystemLoader, PackageLoader, ChoiceLoader
import cherrypy
from cherrypy import thread_data

from cache import Cache

from ion.sqlalchemy_tool import session

__CONTROLLERS__ = []
__CONTROLLERSDICT__ = {}

def route(route, name=None, priority=50):
    def dec(func):
        actual_name = func.__name__
        if name:
            actual_name = name
        conf = (
            actual_name, {
                'route': route,
                'method': func.__name__,
                'priority': priority
            }
        )

        return func, conf

    return dec

def authenticated(func):
    def actual(*arguments, **kw):
        instance = arguments[0]

        instance.server.publish('on_before_user_authentication', {'server':instance, 'context':instance.context})

        user = instance.user
        if user:
            instance.server.publish('on_user_authentication_successful', {'server':instance, 'context':instance.context})
            return func(*arguments, **kw)
        else:
            instance.server.publish('on_user_authentication_failed', {'server':instance, 'context':instance.context})

    actual.__name__ = func.__name__
    actual.__doc__ = func.__doc__
    return actual

class MetaController(type):
    def __init__(cls, name, bases, attrs):
        if 'Controller' in globals() and \
                        issubclass(cls, globals()['Controller']):

            __CONTROLLERS__.append(cls)
            __CONTROLLERSDICT__[name] = cls
            cls.__routes__ = []

            routes = []

            for attr, value in attrs.items():
                if isinstance(value, tuple) and len(value) is 2:
                    method, conf = value
                    routes.append((attr, method, conf, conf[1]['priority']))

            routes = sorted(routes, lambda i1, i2: cmp(i2[3], i1[3]))

            for attr, method, conf, priority in routes:
                setattr(cls, attr, method)
                cls.__routes__.append(conf)

        super(MetaController, cls).__init__(name, bases, attrs)

class Controller(object):
    __metaclass__ = MetaController
    __routes__ = None

    def __init__(self):
        self.server = None

    def log(self, message):
        if self.settings.Ion.as_bool('verbose'):
            cherrypy.log(message, "[%s]" % self.__class__.__name__)

    @classmethod
    def all(self):
        return __CONTROLLERS__

    @property
    def cache(self):
        return self.server.cache

    @property
    def settings(self):
        if not self.server:
            return None
        return self.server.context.settings

    @property
    def context(self):
        if not self.server:
            return None
        return self.server.context

    @property
    def store(self):
        return session

    @property
    def name(self):
        return self.__class__.__name__.lower().replace("controller", "")

    @property
    def user(self):
        try:
            return cherrypy.session.get('authenticated_user', None)
        except AttributeError:
            return None

    def login(self, user):
        cherrypy.session['authenticated_user'] = user

    def logoff(self):
        cherrypy.session['authenticated_user'] = None

    def register_routes(self, dispatcher):
        for route in self.__routes__:
            route_name = "%s_%s" % (self.name, route[0])
            dispatcher.connect(route_name, route[1]["route"], controller=self, action=route[1]["method"])

    def render_template(self, template_file, **kw):
        apps = self.server.apps

        app_loaders = []

        for app in reversed(apps):
            app_loaders.append(PackageLoader(app))

        loader = ChoiceLoader(app_loaders)

        env = Environment(loader=loader)

        for filter_name in self.server.template_filters.keys():
            env.filters[filter_name] = self.server.template_filters[filter_name]

        template = env.get_template(template_file)
        return template.render(user=self.user, settings=self.settings, **kw)

    def send_template_by_mail(self, from_email, to_emails, subject, template_file, html=True, **kw):
        body = self.render_template(template_file, **kw)
        exit_code = self.send_using_sendmail(from_email, to_emails, subject, body, html=html)
        return exit_code

    def send_using_sendmail(self, from_email, to_emails, subject, body, html=True):
        SENDMAIL = "/usr/sbin/sendmail" # sendmail location
        p = os.popen("%s -t" % SENDMAIL, "w")
        p.write("From: %s\n" % from_email)
        p.write("To: %s\n" % (", ".join(to_emails)))
        p.write("Subject: %s\n" % subject)
        p.write("\n") # blank line separating headers from body

        if html:
            p.write('Content-Type: text/html; charset="us-ascii"\n')
            p.write('MIME-Version: 1.0')
            p.write('Content-Transfer-Encoding: 7bit')

        p.write(body)

        sts = p.close()

        if sts != 0:
            self.log("Sendmail exit status: %d" % sts)

        return sts

    def send_using_smtp(self, from_email, to_emails, subject, body):
        pass

    def redirect(self, url):
        raise cherrypy.HTTPRedirect(url)

    def healthcheck(self):
        healthcheck_text = self.settings.Ion.healthcheck_text

        result = self.server.test_connection()
        if not result:
            raise RuntimeError("The connection to the database failed with error: %s" % str(self.server.test_connection_error))

        return healthcheck_text or "WORKING"




#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

Version="0.6.4"
version = Version
__version__ = Version

from context import *
from bus import *
from server import *
from settings import *
from controllers import *
from db import *



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import inspect
import sys
from os.path import join, dirname, splitext, split, exists

import cherrypy

from bus import Bus
from settings import Settings
from fs import imp, locate, is_file

class Context(object):
    def __init__(self, root_dir):
        self.bus = Bus()
        self.settings = Settings(root_dir=root_dir)

    def load_settings(self, config_path):
        self.settings.load(config_path)

    @property
    def apps(self):
        return [app for app in self.settings.Ion.apps.strip().split('\n') if app]

    def load_apps(self):
        self.app_paths = {}
        self.app_modules = {}

        for app in self.apps:
            module = imp(app)
            if not module:
                cherrypy.log.error('Cannot import module from %s.' % app, 'Server')
                sys.exit(0)

            self.app_modules[app] = module
            app_path = dirname(inspect.getfile(self.app_modules[app]))
            self.app_paths[app] = app_path

    def list_all_media(self):
        """docstring for list_all_media"""
        app_media = {}

        for app_path in self.app_paths.values():
            media_path = join(app_path, 'media')
            for file_name in locate("*.txt", "*.py", "*.css", "*.js", "*.rst", "*.html", "*.ini", root=media_path):
                if not is_file(file_name):
                    continue
                key = file_name.replace(media_path, '')
                app_media[key] = file_name
        return app_media



class Form(object):
    pass
    
class Input(object):
    
    def __init__(self, *args, **kwargs):
        self.attrs = kwargs
    
    def render(self):
        return u'<input type="%s" />' % self.type
        
    def __str__(self):
        return self.render()
        
class TextInput(Input):
    type = u'text'
        
class PasswordInput(Input):
    type = u'password'
    
class TextField(object):
    
    def __init__(self, widget=None):
        pass


#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class Bus(object):
    def __init__(self):
        self.subscriptions = {}

    def subscribe(self, subject, func):
        if not subject in self.subscriptions:
            self.subscriptions[subject] = []
        self.subscriptions[subject].append(func)

    def publish(self, subject, data, *args, **kw):
        if subject not in self.subscriptions:
            return

        for subscription in self.subscriptions[subject]:
            subscription(data, *args, **kw)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

from $project_name.main.controllers import *

def fake_render_template(self, template, *args, **kw):
    return "fake rendered text args: %s kw: %s" % (args, kw)

DefaultController.render_template = fake_render_template

def test_default_controller_index_action():
    ctrl = DefaultController()

    result = ctrl.index()
    assert "date" in result
    assert "fake rendered text" in result



#!/usr/bin/env python
#-*- coding:utf-8 -*-

import sys
from os.path import abspath, join, split, dirname

from ion import Server, ServerStatus, Context

class ServerHelper (object):

    def __init__(self, root_path, config_path):
        self.server = Server(root_path)

        self.server.start(config_path, non_block=True)

        while not self.server.status == ServerStatus.Started:
            time.sleep(0.5)

    def stop(self):
        self.server.stop()
        self.server = None

    def ctrl(self, controller):
        _ctrl = controller()
        _ctrl.server = self.server
        return _ctrl




#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from os.path import join, dirname, abspath, exists, isfile
from os import walk, remove

import fnmatch
import shutil

open_file = open

#should be called like: locate('*.txt', '*.py', '*.acc', root=path)
def locate(*args, **kw):
    root = 'root' in kw and kw['root'] or os.curdir
    root_path = abspath(root)
    patterns = args

    return_files = []
    for path, dirs, files in walk(root_path):
        for pattern in args:
            for filename in fnmatch.filter(files, pattern):
                return_files.append(join(path, filename))
    return return_files

def recursive_copy(from_path, to_path):
    shutil.copytree(from_path, to_path)

def move_dir(from_path, to_path):
    shutil.move(from_path, to_path)

def read_all_file(path):
    project_file = open_file(path, 'r')
    text = project_file.read()
    project_file.close()

    return text

def replace_file_contents(path, contents):
    project_file = open_file(path, 'w')
    project_file.write(contents)
    project_file.close()

def remove_file(path):
    remove(path)

def is_file(path):
    return isfile(path)

def imp(name, import_method=None):
    if not import_method:
        import_method = __import__
    try:
        module = import_method(name)
        if "." in name:
            return reduce(getattr, name.split('.')[1:], module)
        return module
    except ImportError:
        return None


from datetime import datetime

from ion.controllers import Controller, route

class DefaultController(Controller):

    @route('/')
    def index(self):
        return self.render_template("index.html", date=datetime.now())



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sqlalchemy import MetaData, create_engine, __version__ as sa_version
from sqlalchemy.orm import scoped_session, sessionmaker

if sa_version.split(".") < ["0", "5", "0"]:
    raise ImportError("Version 0.5 or later of SQLAlchemy required.")

def new_session():
    return scoped_session(sessionmaker(autoflush=True, autocommit=False))

metadata = MetaData()
session = new_session()
mapper = session.mapper

class Db(object):
    def __init__(self, server):
        self.is_connected = False
        self.server = server
        self.store = None

    def connect(self):
        if self.is_connected:
            raise RuntimeError("You have to disconnect before connecting again")
        self.is_connected = True

        engine = create_engine(self.connstr, echo=False, convert_unicode=True)
        session.bind = engine

        self.store = session

    def disconnect(self):
        if not self.is_connected:
            raise RuntimeError("You have to connect before disconnecting")

        self.store.close()
        self.store = None
        self.is_connected = False

    @property
    def connstr(self):
        protocol = self.server.context.settings.Db.protocol
        username = self.server.context.settings.Db.user
        password = self.server.context.settings.Db.password
        host = self.server.context.settings.Db.host
        port = int(self.server.context.settings.Db.port)
        database = self.server.context.settings.Db.database

        return "%s://%s:%s@%s:%d/%s" % (
            protocol,
            username,
            password,
            host,
            port,
            database
        )



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ion import Version
from setuptools import setup, find_packages

setup(
    name = 'Ion',
    version = Version,
    description = "Ion is an MVC Web Framework",
    long_description = """Ion is an MVC Web Framework.""",
    keywords = 'MVC Web Framework',
    author = 'Bernardo Heynemann',
    author_email = 'heynemann@gmail.com',
    url = 'http://www.ionwebframework.org',
    license = 'OSI',
    classifiers = ['Development Status :: 4 - Beta',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved',
                   'Natural Language :: English',
                   'Operating System :: MacOS',
                   'Operating System :: Microsoft :: Windows',
                   'Operating System :: POSIX :: Linux',
                   'Programming Language :: Python :: 2.6',],
    packages = ['ion'],
    include_package_data=True,
    package_data = {
        '': ['*.txt'],
        '': ['*.rst'],
        '': ['*.html'],
        '': ['*.py'],
    },
    install_requires=[
        "cherrypy",
        "jinja2",
        "sqlalchemy",
        "routes",
        "nose"
    ],
    entry_points = {
        'console_scripts': [
            'ion = ion.console.ion_console:main',
        ],
    },
)




#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import cherrypy
from ion.db import Db

class Service(object):
    def __init__(self, server):
        self.server = server
        self.db = None
        self.store = None

    def log(self, message):
        if self.server.context.settings.Ion.as_bool('verbose'):
            cherrypy.log(message, "[%s]" % self.__class__.__name__)

    def connect(self):
        db = Db(self.server)
        db.connect()
        self.db = db
        self.store = db.store

    def disconnect(self):
        self.db.disconnect()
        self.db = None
        self.store = None



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# Copyright Bernardo Heynemann <heynemann@gmail.com>

# Licensed under the Open Software License ("OSL") v. 3.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.opensource.org/licenses/osl-3.0.php

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys
import os
from os.path import join, dirname, abspath, exists, split
from string import Template
import inspect
import shutil

from ion.server import Server
from ion.fs import *
from ion import Version, Context

def log(message):
    print message

__PROVIDERS__ = []
__PROVIDERSDICT__ = {}

class MetaProvider(type):
    def __init__(cls, name, bases, attrs):
        if name not in ('MetaProvider', 'Provider'):
            __PROVIDERS__.append(cls)
            __PROVIDERSDICT__[name] = cls

        super(MetaProvider, cls).__init__(name, bases, attrs)

class Provider(object):
    __metaclass__ = MetaProvider

    @classmethod
    def all(self):
        return __PROVIDERS__

    def __init__(self, key):
        self.key = key

class VersionProvider(Provider):
    def __init__(self):
        super(VersionProvider, self).__init__("version")

    def execute(self, current_dir, options, args):
        log("Ion v%s - http://github.com/heynemann/ion\nCreated by Bernardo Heynemann (heynemann@gmail.com)" % Version)

class HelpProvider(Provider):
    def __init__(self):
        super(HelpProvider, self).__init__("help")

    def execute(self, current_dir, options, args):
        log("Help to be written")

class CreateProjectProvider(Provider):
    def __init__(self):
        super(CreateProjectProvider, self).__init__("create")

    def replace_tokens(self, path, **kw):
        for file_name in locate("*.txt", "*.py", "*.css", "*.js", "*.rst", "*.html", "*.ini", root=path):
            if not is_file(file_name):
                continue
            text = read_all_file(file_name)

            remove_file(file_name)

            s = Template(text)

            contents = s.substitute(**kw)
            replace_file_contents(file_name, contents)

    def execute(self, current_dir, options, args):
        if not args or not args[0]:
            raise ValueError(" You need to pass the project name to be created")

        project_name = args[0]

        new_project_template = abspath(join(dirname(__file__), "new_project"))
        to_project_path = abspath(join(current_dir, project_name))

        if exists(to_project_path):
            raise ValueError("The choosen path(%s) already exists! Please choose a different name or try another folder." % to_project_path)

        recursive_copy(new_project_template, to_project_path)

        self.replace_tokens(to_project_path, project_name=project_name)

        move_dir(join(to_project_path, "src"), join(to_project_path, project_name))

class RunServerProvider(Provider):
    def __init__(self):
        super(RunServerProvider, self).__init__("run")

    def execute(self, current_dir, options, args):
        ini_files = locate("config.ini", root=current_dir)

        if not ini_files:
            raise RuntimeError("No files called config.ini were found in the current directory structure")

        root_dir = abspath(dirname(ini_files[0]))

        sys.path.append(os.curdir)
        sys.path.append(root_dir)

        server = Server(root_dir=root_dir)

        try:
            server.start("config.ini")
        except KeyboardInterrupt:
            server.stop()

class TestRunnerProvider(Provider):
    def __init__(self, key=None):
        super(TestRunnerProvider, self).__init__(key or "test")

    def run_nose(self, apps, config_path, paths, project_name):
        from nose.core import run
        from nose.config import Config

        argv = ["-d", "-s", "--verbose"]

        use_coverage = True
        try:
            import coverage
            argv.append("--with-coverage")
            argv.append("--cover-erase")
            #argv.append("--cover-package=%s" % project_name)
            argv.append("--cover-inclusive")
        except ImportError:
            pass

        if exists(config_path):
            argv.append("--config=%s" % config_path)

        for path in paths:
            argv.append(path)
            
        if use_coverage:
            for app in apps:
                argv.append("--cover-package=%s" % app)

        result = run(argv=argv)
        if not result:
            sys.exit(1)

    def execute(self, current_dir, options, args, complement_dir=None):
        config_file = locate('config.ini')
        if not config_file:
            raise RuntimeError('Could not find config.ini file in this project, thus can\'t run unit tests')
        config_file = config_file[0]

        root_dir = dirname(config_file)
        
        context = Context(root_dir)
        context.load_settings('config.ini')

        sys.path.append(os.curdir)
        sys.path.append(root_dir)
        
        tests_dirs = []
        for app in context.apps:
            module = imp(app)

            if not module:
                log('Cannot import module [%s]' % app)
                sys.exit(0)

            module_path = dirname(inspect.getfile(module))
            
            if complement_dir:
                tests_dirs.append(join(module_path, "tests", complement_dir))
            else:
                tests_dirs.append(join(module_path, "tests"))

        self.run_nose(context.apps,
                      join(dirname(config_file), 'nose.cfg'),
                      tests_dirs, 
                      os.path.split(current_dir)[-1])

class UnitTestProvider(TestRunnerProvider):
    def __init__(self):
        super(UnitTestProvider, self).__init__("unit")

    def execute(self, current_dir, options, args):
        super(UnitTestProvider, self).execute(current_dir, options, args, 'unit')

class FunctionalTestProvider(TestRunnerProvider):
    def __init__(self):
        super(FunctionalTestProvider, self).__init__("func")

    def execute(self, current_dir, options, args):
        super(FunctionalTestProvider, self).execute(current_dir, options, args, 'functional')

class PackageMediaProvider(Provider):
    def __init__(self):
        super(PackageMediaProvider, self).__init__("package_media")

    def execute(self, current_dir, options, args):
        if not args:
            msg = 'You must specify the path to drop the files at' + \
                  ' as an argument.'
            raise RuntimeError(msg)

        target_dir = args[0]

        ini_files = locate("config.ini", root=current_dir)

        if not ini_files:
            raise RuntimeError("No files called config.ini were found" + \
                               "in the current directory structure")

        root_dir = abspath(dirname(ini_files[0]))

        context = Context(root_dir)
        context.load_settings(ini_files[0])
        context.load_apps()
        
        medias = context.list_all_media()
        
        if not exists(target_dir):
            os.mkdir(target_dir)
        
        for media in medias.keys():
            filename = join(target_dir, media.strip("/"))
            if not exists(dirname(filename)):
                os.makedirs(dirname(filename))
            shutil.copyfile(medias[media], filename)
        
        print "All files properly packaged."